package core.services;

import core.api.models.timemanager.taskgroup.TaskgroupEntityInfo;
import core.api.models.timemanager.taskgroup.TaskgroupFieldInfo;
import core.entities.User;
import core.entities.UserRole;
import core.entities.timemanager.Taskgroup;
import core.repositories.UserRepository;
import core.repositories.timemanager.TaskgroupRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.config.Task;
import org.springframework.security.core.parameters.P;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.NoSuchElementException;
import java.util.Optional;

@Service
public class TaskgroupService {

    private final TaskgroupRepository taskgroupRepository;
    private final UserRepository userRepository;
    public TaskgroupService(@Autowired  TaskgroupRepository taskgroupRepository,
                            @Autowired UserRepository userRepository) {
        this.taskgroupRepository = taskgroupRepository;
        this.userRepository = userRepository;
    }

    public PermissionResult<Taskgroup> getTaskgroupByIDAndUsername(long taskgroupID, String username) {
        Optional<Taskgroup> taskgroup = taskgroupRepository.findById(taskgroupID);
        return taskgroup.map(value -> new PermissionResult<>(value, value.getUser().getUsername().equals(username))).orElseGet(() ->
                new PermissionResult<>(ServiceExitCode.MISSING_ENTITY));
    }

    public ServiceResult<Taskgroup> addTaskgroup(TaskgroupFieldInfo taskData, String username) {
        Optional<User> user = userRepository.findByUsername(username);
        if(user.isEmpty()) {
            throw new NoSuchElementException();
        }

        Taskgroup taskgroup;
        if(taskData.getParentID() < 0) {
            if(existTaskgroupOnTaskgroupLevel(null, taskData.getName(), username)) {
                return new ServiceResult<>(ServiceExitCode.ENTITY_ALREADY_EXIST);
            } else {
                taskgroup = new Taskgroup(taskData.getName(), user.get());
            }

        } else{
            Optional<Taskgroup> parentTaskgroup = taskgroupRepository.findById(taskData.getParentID());
            if(parentTaskgroup.isEmpty()) {
                return new ServiceResult<>(ServiceExitCode.MISSING_ENTITY);
            } else if(existTaskgroupOnTaskgroupLevel(parentTaskgroup.get(), taskData.getName(), username)) {
                return new ServiceResult<>(ServiceExitCode.ENTITY_ALREADY_EXIST);
            } else {
                taskgroup = new Taskgroup(taskData.getName(), user.get());
                taskgroup.setParent(parentTaskgroup.get());
                parentTaskgroup.get().getChildren().add(taskgroup);
            }
        }
        taskgroupRepository.save(taskgroup);
        return new ServiceResult<>(taskgroup);
    }

    public ServiceExitCode editTaskgroup(Taskgroup taskgroup, TaskgroupFieldInfo taskgroupFieldInfo) {
        if(!taskgroup.getTaskgroupName().equals(taskgroupFieldInfo.getName())) {
            //Check if another taskgroup with the new name is already existent for the user of the taskgroup
            if(!existTaskgroupOnTaskgroupLevel(taskgroup.getParent(), taskgroupFieldInfo.getName(), taskgroup.getUser().getUsername())) {
                taskgroup.setTaskgroupName(taskgroupFieldInfo.getName());
                taskgroupRepository.save(taskgroup);
                return ServiceExitCode.OK;
            } else {
                return ServiceExitCode.ENTITY_ALREADY_EXIST;
            }
        } else {
            return ServiceExitCode.OK;
        }
    }

    private boolean existTaskgroupOnTaskgroupLevel(Taskgroup taskgroup, String taskgroupName, String username) {
        if(taskgroup == null) {
            List<Taskgroup> topTaskgroups = getTopTaskgroupsByUser(username);
            for(Taskgroup taskgroup1 : topTaskgroups) {
                if(taskgroup1.getTaskgroupName().equals(taskgroupName)) {
                    return true;
                }
            }
        } else {
            for(Taskgroup sibling : taskgroup.getChildren()) {
                if(sibling.getTaskgroupName().equals(taskgroupName)) {
                    return true;
                }
            }
        }
        return false;
    }

    public void deleteTaskgroup(Taskgroup taskgroup) {
        taskgroupRepository.delete(taskgroup);
    }

    public List<Taskgroup> getTopTaskgroupsByUser(String username) {
        return taskgroupRepository.findAllTopTaskgroupsByUser(username);
    }

    public void deleteTaskgroupByUser(User user) {
        taskgroupRepository.deleteAllByUser(user);
    }
}
